const path = require('path');
const child_process = require('child_process');
const Promise = require('bluebird');

const { ZMQ_WORKER_MESSAGE } = require('enums').OBJECT_TYPE;
const { sendEmail } = require('mail');
const Sms = require('sms');

const logger = require('../utils/logger');
const { safeSendZmqOnce } = require('../routes/mp_messaging');
const gl = require('../routes/global');
const { public_url, low_battery_notification } = require('../routes/config');
const Notifications = require('../routes/notifications');
const Subscribers = require('../routes/subscribers');

const { db } = gl;
const PROJECT_ALL = 'project_uid';
/**
 * use battery.pr < 20
 * cron
 */

function getNodes() {
    return new Promise((resolve) => {
        const message = {
            type: ZMQ_WORKER_MESSAGE.STATE_REQUEST,
            request: {
                node_request: {
                    node_low_battery: true,
                },
            },
        };

        if (!gl.childZMQ) {
            resolve({});
            return;
        }
        safeSendZmqOnce(gl.childZMQ, message, (mpState) => {
            resolve(mpState);
        });
    });
}

function getSmsBody(count, url) {
    return `Low battery on ${count} tag${count > 1 ? 's' : ''}. Go to ${url} for more information.`;
}

function getEmailBody({ data, public_url_, formattedMessage }) {
    return {
        from: '"Redpoint Positioning" <lab@redpointpositioning.com>', // sender address
        to: data.email, // list of receivers
        subject: 'Low battery alarm report',
        html: `
            <article style='border-radius: 2px;padding: 20px;background-color: rgb(255,255,255);border-bottom: 1px solid #b30000;margin: 14px;'>
                <h2><span style='color:#b30000'>Redpoint</span> Positioning</h2>
                <h3>Hello ${data.name}, you have a low battery alarm report.</h3>
                <p>Server URL: ${public_url_}</p>
                <p>${formattedMessage}</p>
                <a href=${public_url_}/#/>Click to sign in to SiteWise</a>
            </article>`, // html body
    };
}

function getTableBody(nodes) {
    return nodes.map((node) => (`
    <tr>
        <th style="border: 1px solid #d4d4d4">${node.mac_address}</th>
        <th style="border: 1px solid #d4d4d4">${node.trackable_name || '-'}</th>
        <th style="border: 1px solid #d4d4d4">${node.battery}</th>
    </tr>`)).join('');
}

function getFormattedMessage(nodes) {
    return `
    <table style="border: 1px solid #d4d4d4; border-collapse: collapse; width: 100%;">
        <thead style="background-color: #f1f1f1!important">
            <tr>
                <th style="border: 1px solid #d4d4d4">MAC address</th>
                <th style="border: 1px solid #d4d4d4">Trackable</th>
                <th style="border: 1px solid #d4d4d4">Battery level, %</th>
            </tr>
        </thead>
        <tbody>
            ${getTableBody(nodes)}
        </tbody>
    </table>`;
}

async function sendingEmails(projectData) {
    const formattedMessage = getFormattedMessage(projectData.nodes);

    projectData.emailList.forEach(async (x) => {
        const body = getEmailBody({
            data: { name: x.name, email: x.email },
            public_url_: public_url,
            formattedMessage,
        });
        // logger.info('[Low Battery checking script] Sending Email: ', body);
        await sendEmail(body);
    });
    return undefined;
}

async function sendingSms(projectData) {
    const text = getSmsBody(projectData.nodes.length, public_url);
    projectData.smsList.forEach(async (number) => {
        // logger.info(`[Low Battery checking script] Sending SMS "${text}" to number "${text}"`);
        await Sms.sendSms(number, text);
    });
    return undefined;
}

function handler({ nodes, notifications, subscribers }) {
    logger.info('[Low Battery checking script] START');
    const worker = path.join(__dirname, 'low_battery_notify', 'worker.js');
    const child = child_process.fork(worker);
    child.on('error', (err) => {
        logger.error('error');
        logger.logex(err);
    });
    child.on('message', (msg) => {
        const nodesWithLowBattery = JSON.parse(msg);
        logger.verbose('nodesWithLowBattery', nodesWithLowBattery);
        Object.keys(nodesWithLowBattery).forEach(async (key) => {
            const projectData = nodesWithLowBattery[key];
            if (projectData.nodes.length > 0) {
                if (projectData.emailList.length > 0) {
                    await sendingEmails(projectData);
                }
                if (projectData.smsList.length > 0) {
                    await sendingSms(projectData);
                }
            }
        });
        // logger.info('message', msg);
    });
    child.on('exit', () => {
        logger.info('[Low Battery checking script] STOP');
    });
    child.send({ nodes, notifications, subscribers });
}

function execute() {
    let interval;
    if (low_battery_notification.sending) {
        logger.info(`[Low Battery checking script] Start. Run every ${low_battery_notification.minutes_timeout} minutes`);
        if (!interval) {
            interval = setInterval(() => {
                Promise.join(
                    getNodes(),
                    db.queryAsync('SELECT * FROM view_notifications WHERE conditions LIKE \'%node_low_battery%\' and active = 1;')
                        .map(Notifications.formatNotification),
                    Subscribers.getSubscribers({ uid: PROJECT_ALL }),
                )
                    .then(([nodes, notifications, subscribers]) => {
                        handler({ ...nodes, notifications, subscribers });
                    })
                    .catch((err) => {
                        logger.error('[Low Battery checking script] FAILS');
                        logger.logex(err);
                        interval = null;
                    });
            }, low_battery_notification.minutes_timeout * 60000);
        }
    } else {
        logger.info('[Low Battery checking script] not working');
    }
}

module.exports = {
    execute,
};
